# Generated by Django 4.2.17 on 2024-12-23 07:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_app_metadata", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="config",
            name="key",
            field=models.CharField(
                max_length=64, unique=True, verbose_name="配置项标识"
            ),
        ),
    ]
